from ADR.Components.Aerodynamic_components.Aerodynamic_surface import (
    Aerodynamic_surface,
)


class VS(Aerodynamic_surface):
    def __init__(self, data):
        super().__init__(data)

        self.calc_area()

    def calc_area(self):
        self.area = self.section1.area + self.section2.area



from math import radians, cos, sin


class SM:
    """Class used to calculate the static margin
     for a given plane flying on a given alpha"""

    def __init__(self, plane_type, wing1, wing2, hs, alpha_plane, CM):

        self.SM = 0

        eta = 0.9
        den = (
            wing1.dCL_dalpha.at[wing1.attack_angle, "CL"] * cos(radians(alpha_plane))
            + wing1.dCD_dalpha.at[wing1.attack_angle, "CD"] * sin(radians(alpha_plane))
            + (
                hs.dCL_dalpha.at[hs.attack_angle, "CL"] * cos(radians(alpha_plane))
                + hs.dCD_dalpha.at[hs.attack_angle, "CD"] * sin(radians(alpha_plane))
            )
            * eta
            * hs.area
            / wing1.area
        )

        if plane_type == "biplane":
            den += (
                wing2.dCL_dalpha.at[wing2.attack_angle, "CL"]
                * cos(radians(alpha_plane))
                * wing2.area
                / wing1.area
                + wing2.dCD_dalpha.at[wing2.attack_angle, "CD"]
                * sin(radians(alpha_plane))
                * wing2.area
                / wing1.area
            )
            self.SM = (
                -CM / den
            )  # Colocar a corda media aerodinamica e nao a chorda da raiz.
        elif plane_type == "monoplane":
            self.SM = (
                -CM / den
            )  # Colocar a corda media aerodinamica e nao a chorda da raiz.
        else:
            print(
                "------ Error: Incapable of analysing Static Margin of given plane ------"
            )



class Component:
    def __init__(self, data):
        self.x = data.get("x")
        self.y = data.get("y")
        self.z = data.get("z")

        self.tag = ""

    def __str__(self):
        return self.__class__.__name__



from math import sin, cos, radians, degrees
import pandas as pd

from ADR.Methods.FundamentalEquations import drag


class Takeoff:
    def __init__(self, plane, performance_parameters):

        self.plane = plane
        self.rho_air = performance_parameters.get("rho_air")
        self.dist_max = performance_parameters.get("dist_max")
        self.offset_pilot = performance_parameters.get("offset_pilot")

        self.distx_wing1_tpr = abs(plane.wing1.ca.abs_x - plane.tpr.x)
        self.distz_wing1_tpr = abs(plane.wing1.ca.abs_z - plane.tpr.z)

        self.distx_wing2_tpr = abs(plane.wing2.ca.abs_x - plane.tpr.x)
        self.distz_wing2_tpr = abs(plane.wing2.ca.abs_z - plane.tpr.z)

        self.distx_hs_tpr = abs(plane.hs.ca.abs_x - plane.tpr.x)
        self.distz_hs_tpr = abs(plane.hs.ca.abs_z - plane.tpr.z)

        self.distx_cg_tpr = abs(plane.cg.x - plane.tpr.x)
        self.distz_cg_tpr = abs(plane.cg.z - plane.tpr.z)

        self.distx_motor_tpr = abs(plane.motor.x - plane.tpr.x)
        self.distz_motor_tpr = abs(plane.motor.z - plane.tpr.z)

    def get_mtow(self):

        m = 1  # Massa total inicial do aviao [kg]
        g = 9.81  # Constante gravitacional [m/s^2]

        dt = 0.01  # Incremento discreto de tempo [s]
        dm = 0.1  # Incremento de massa [kg]

        incidence_active_hs = (
            # Angulo de incidencia adicionado no profundor ao ser acionado [deg]
            10
        )

        takeoff_failed = False

        self.mass_dict = {}

        while not takeoff_failed:
            m = m + dm
            self.plane.mtow = m

            theta_airplane_deg = 0  # Angulo do aviao com a pista [°]
            V_x = 0  # Velocidade inicial do aviao no eixo X [m/s]
            pilot_triggered = False  # O piloto acionou o profundor?

            dist_x = 0  # Distancia percorrida em X [m]
            N = 0.1  # Força normal [N]
            t = 0  # Tempo [s]

            incidence_w1 = 0  # Angulo de incidencia da asa1 [deg]
            incidence_w2 = 0  # Angulo de incidencia da asa2 [deg]
            incidence_hs = 0  # Angulo de incidencia do profundor [deg]

            on_ground = True
            takeoff_failed = False
            going_forward = True

            time_dict = {}

            while on_ground and going_forward and not takeoff_failed:

                alpha_w1 = theta_airplane_deg + incidence_w1
                if self.plane.plane_type == "biplane":
                    alpha_w2 = theta_airplane_deg + incidence_w2
                alpha_hs = theta_airplane_deg + incidence_hs

                E = self.plane.motor.thrust(V_x)

                t = t + dt

                L_w1 = self.plane.wing1.lift(self.rho_air, V_x, alpha_w1)
                L_w2 = 0  # Value if there's no wing2
                if self.plane.plane_type == "biplane":
                    L_w2 = self.plane.wing2.lift(self.rho_air, V_x, alpha_w2)
                L_hs = self.plane.hs.lift(self.rho_air, V_x, alpha_hs)
                L = L_w1 + L_w2 - L_hs

                E_z = E * sin(radians(theta_airplane_deg))
                W = m * g

                N = W - L - E_z

                E_x = E * cos(radians(theta_airplane_deg))

                D_w1 = self.plane.wing1.drag(self.rho_air, V_x, alpha_w1)
                D_w2 = 0  # Value if there's no wing2
                if self.plane.plane_type == "biplane":
                    D_w2 = self.plane.wing2.drag(self.rho_air, V_x, alpha_w2)
                D_hs = self.plane.hs.drag(self.rho_air, V_x, alpha_hs)
                D_tp = drag(self.rho_air, V_x,
                            self.plane.S_tp, self.plane.CD_tp)
                D_fus = drag(self.rho_air, V_x,
                             self.plane.S_fus, self.plane.CD_fus)
                D = D_w1 + D_w2 + D_hs + D_tp + D_fus

                F_at = self.plane.u_k * N

                F_x = E_x - D - F_at
                dV_x = ((F_x) / m) * dt
                V_x = V_x + dV_x
                dist_x = dist_x + V_x * dt

                M_w1 = self.plane.wing1.moment(self.rho_air, V_x, alpha_w1)
                M_w2 = 0  # Value if there's no wing2
                if self.plane.plane_type == "biplane":
                    M_w2 = self.plane.wing2.moment(self.rho_air, V_x, alpha_w2)
                M_hs = self.plane.hs.moment(self.rho_air, V_x, alpha_hs)

                M_x = (
                    E_z * self.distx_motor_tpr
                    - W * self.distx_cg_tpr
                    + L_w1 * self.distx_wing1_tpr
                    + L_w2 * self.distx_wing2_tpr
                    + L_hs * self.distx_hs_tpr
                )
                M_z = (
                    -E_x * self.distz_motor_tpr
                    + D_w1 * self.distz_wing1_tpr
                    + D_w2 * self.distz_wing2_tpr
                    + D_hs * self.distz_hs_tpr
                )
                M = M_x + M_z - M_hs + M_w1 + M_w2
                dOmega = (M / self.plane.Iyy_TPR) * dt
                dTheta = dOmega * dt

                if theta_airplane_deg + degrees(dTheta) >= 0:
                    theta_airplane_deg = theta_airplane_deg + degrees(dTheta)

                if (
                    self.dist_max - dist_x
                ) <= self.offset_pilot and pilot_triggered == False:
                    incidence_hs += incidence_active_hs
                    pilot_triggered = True
                    alpha_hs = theta_airplane_deg + incidence_hs

                if dist_x > self.dist_max:
                    takeoff_failed = True
                else:
                    takeoff_failed = False

                if N > 0:
                    on_ground = True
                else:
                    on_ground = False

                    V_takeoff = V_x
                    V_stall = self.plane.get_V_stall(self.rho_air)

                    if V_takeoff < V_stall or D > E_x:
                        takeoff_failed = True

                    self.plane.V_takeoff = V_takeoff

                if dist_x > -5:
                    going_forward = True
                else:
                    going_forward = False
                    takeoff_failed = True
                    print("Indo pra tras")

                time_data = [
                    theta_airplane_deg,
                    E,
                    L,
                    L_w1,
                    L_w2,
                    L_hs,
                    D,
                    D_w1,
                    D_w2,
                    D_hs,
                    N,
                    F_at,
                    V_x,
                    dist_x,
                    M,
                    M_w1,
                    M_w2,
                    M_hs,
                    dTheta,
                    incidence_hs,
                ]
                time_dict[t] = time_data

            time_df = pd.DataFrame.from_dict(
                time_dict,
                orient="index",
                columns=[
                    "theta",
                    "E",
                    "L",
                    "L_w1",
                    "L_w2",
                    "L_hs",
                    "D",
                    "D_w1",
                    "D_w2",
                    "D_hs",
                    "N",
                    "F_at",
                    "V_x",
                    "dist_x",
                    "M",
                    "M_w1",
                    "M_w2",
                    "M_hs",
                    "dTheta",
                    "incidence_hs",
                ],
            )
            time_df.index.name = "t"
            self.mass_dict[m] = time_df

        return self.plane.mtow



from ADR.Components.Aerodynamic_components.Wing import Wing
from ADR.Components.Aerodynamic_components.HS import HS
from ADR.Components.Propulsion.Motor import Motor
from ADR.Components.Points.CG import CG
from ADR.Components.Points.TPR import TPR
from ADR.Core.data_manipulation import dict_to_dataframe

import numpy as np


class Plane:
    def __init__(self, data):
        self.data = data
        self.plane_type = data.get("plane_type")

        wing1_data = {
            "x": data.get("wing1_x"),
            "y": data.get("wing1_y"),
            "z": data.get("wing1_z"),
            "airfoil_clmax": data.get("wing1_clmax_airfoil"),
            "airfoil1_name": data.get("wing1_airfoil1_name"),
            "airfoil2_name": data.get("wing1_airfoil2_name"),
            "airfoil3_name": data.get("wing1_airfoil3_name"),
            "span1": data.get("wing1_span1"),
            "span2": data.get("wing1_span2"),
            "chord1": data.get("wing1_chord1"),
            "chord2": data.get("wing1_chord2"),
            "chord3": data.get("wing1_chord3"),
            "twist1": data.get("wing1_twist1"),
            "twist2": data.get("wing1_twist2"),
            "twist3": data.get("wing1_twist3"),
            "incidence": data.get("wing1_incidence"),
            "CM_ca": data.get("wing1_CM_ca"),
        }

        wing2_data = {
            "x": data.get("wing2_x"),
            "y": data.get("wing2_y"),
            "z": data.get("wing2_z"),
            "airfoil_clmax": data.get("wing2_clmax_airfoil"),
            "airfoil1_name": data.get("wing2_airfoil1_name"),
            "airfoil2_name": data.get("wing2_airfoil2_name"),
            "airfoil3_name": data.get("wing2_airfoil3_name"),
            "span1": data.get("wing2_span1"),
            "span2": data.get("wing2_span2"),
            "chord1": data.get("wing2_chord1"),
            "chord2": data.get("wing2_chord2"),
            "chord3": data.get("wing2_chord3"),
            "twist1": data.get("wing2_twist1"),
            "twist2": data.get("wing2_twist2"),
            "twist3": data.get("wing2_twist3"),
            "incidence": data.get("wing2_incidence"),
            "CM_ca": data.get("wing2_CM_ca"),
        }

        hs_data = {
            "x": data.get("hs_x"),
            "y": data.get("hs_y"),
            "z": data.get("hs_z"),
            "airfoil_clmax": data.get("hs_clmax_airfoil"),
            "airfoil1_name": data.get("hs_airfoil1_name"),
            "airfoil2_name": data.get("hs_airfoil2_name"),
            "airfoil3_name": data.get("hs_airfoil3_name"),
            "span1": data.get("hs_span1"),
            "span2": data.get("hs_span2"),
            "chord1": data.get("hs_chord1"),
            "chord2": data.get("hs_chord2"),
            "chord3": data.get("hs_chord3"),
            "twist1": data.get("hs_twist1"),
            "twist2": data.get("hs_twist2"),
            "twist3": data.get("hs_twist3"),
            "incidence": data.get("hs_incidence"),
            "CM_ca": data.get("hs_CM_ca"),
        }

        motor_data = {
            "x": data.get("motor_x"),
            "y": data.get("motor_y"),
            "z": data.get("motor_z"),
            "static_thrust": data.get("static_thrust"),
            "linear_decay_coefficient": data.get("linear_decay_coefficient"),
        }

        cg_data = {"x": data.get("cg_x"), "z": data.get("cg_z")}

        tpr_data = {"x": data.get("tpr_x"), "z": data.get("tpr_z")}

        self.Iyy_TPR = data.get("Iyy_TPR")
        self.CD_tp = data.get("CD_tp")
        self.S_tp = data.get("S_tp")
        self.CD_fus = data.get("CD_fus")
        self.S_fus = data.get("S_fus")
        self.u_k = data.get("u_k")

        self.wing1 = Wing(wing1_data)
        self.wing2 = Wing(wing2_data)
        self.hs = HS(hs_data)
        # self.vs = VS(vs_data)
        self.motor = Motor(motor_data)
        self.cg = CG(cg_data)
        self.tpr = TPR(tpr_data)

        self.V_stall = 0
        self.V_min = 0
        self.V_max = 0
        self.V_takeoff = 0
        self.mtow = 5
        self.alpha_min = 0
        self.alpha_max = 0
        self.alpha_trimm_min = 0
        self.alpha_trimm_max = 0
        self.tail_trimm = 0

        self.SM_alpha = None

        self.trimm_for_low_angles = False
        self.trimm_for_high_angles = False
        self.positive_sm_for_positive_alphas = False
        self.dimensions_are_good = False

        self.total_dimensions = 0
        self.dead_weight = 0
        self.payload = 0
        self.score = None

        self.dead = False

        self.get_CL_alpha_plane()
        self.get_CD_alpha_plane()
        self.set_alpha_range()
        self.hs.set_incidence_range(self.stall_min, self.stall_max)

    def __str__(self):
        return self.__class__.__name__

    def set_alpha_range(self):
        wings_stall_min = max(self.wing1.stall_min, self.wing2.stall_min)
        wings_stall_max = min(self.wing1.stall_max, self.wing2.stall_max)

        # incidence_min = min(self.wing1.incidence, self.wing2.incidence)
        # incidence_max = max(self.wing1.incidence, self.wing2.incidence)
        # TODO: Incidence for now is fixed on 0 and should be better implemented

        self.stall_min = wings_stall_min
        self.stall_max = wings_stall_max

        self.alpha_range = np.arange(self.stall_min, self.stall_max + 1)

    def set_alpha_trimmed(self, alpha_airplane):
        self.wing1.update_alpha(alpha_airplane)
        if self.plane_type == "biplane":
            self.wing2.update_alpha(alpha_airplane)
        hs_incidence = np.interp(
            alpha_airplane,
            self.tail_trimm.index.values,
            self.tail_trimm["hs_incidence"],
        )
        self.hs.incidence = hs_incidence
        self.hs.update_alpha(alpha_airplane)

    def get_CL_alpha_plane(self):
        CL_alpha_plane = {}
        for alpha in np.arange(-10, 21, 1.0):
            numerator = (
                self.wing1.get_CL(alpha) * self.wing1.area
                - self.hs.get_CL(alpha) * self.hs.area
            )
            if self.plane_type == "biplane":
                numerator += self.wing2.get_CL(alpha) * self.wing2.area
            CL_alpha_plane[alpha] = numerator / self.wing1.area
        self.CL_alpha = dict_to_dataframe(CL_alpha_plane, "CL", "alpha")
        return self.CL_alpha

    def get_CD_alpha_plane(self):
        CD_alpha_plane = {}
        for alpha in np.arange(-10, 21, 1.0):
            numerator = (
                self.wing1.get_CD(alpha) * self.wing1.area
                - self.hs.get_CD(alpha) * self.hs.area
            )
            if self.plane_type == "biplane":
                numerator += self.wing2.get_CD(alpha) * self.wing2.area
            CD_alpha_plane[alpha] = numerator / self.wing1.area
        self.CD_alpha = dict_to_dataframe(CD_alpha_plane, "CD", "alpha")
        return self.CD_alpha

    def get_V_stall(self, rho):
        self.CL_max = self.CL_alpha.max()[0]
        self.V_stall = (
            (2 * self.mtow * 9.81) / (rho * self.wing1.area * self.CL_max)
        ) ** 0.5
        return self.V_stall

    def get_V_CLmin(self, rho):
        self.CL_min = self.CL_alpha.min()[0]
        self.V_CLmin = (
            (2 * self.mtow * 9.81) / (rho * self.wing1.area * self.CL_min)
        ) ** 0.5
        return self.V_CLmin

    def show_plane(self):
        pass
        # print('-------------------------------------------------------------')

        # print("\nPlane components:\n")

        # print("\t--- ", self.wing1, " ---")
        # "x": data.get("wing1_x"),
        # "y": data.get("wing1_y"),
        # "z": data.get("wing1_z"),
        # "airfoil_clmax": data.get("wing1_clmax_airfoil"),
        # "airfoil1_name": data.get("wing1_airfoil1_name"),
        # "airfoil2_name": data.get("wing1_airfoil2_name"),
        # "airfoil3_name": data.get("wing1_airfoil3_name"),
        # "span1": data.get("wing1_span1"),
        # "span2": data.get("wing1_span2"),
        # "chord1": data.get("wing1_chord1"),
        # "chord2": data.get("wing1_chord2"),
        # "chord3": data.get("wing1_chord3"),
        # "twist1": data.get("wing1_twist1"),
        # "twist2": data.get("wing1_twist2"),
        # "twist3": data.get("wing1_twist3"),
        # "incidence": data.get("wing1_incidence"),
        # "CM_ca": data.get("wing1_CM_ca"),
        # print()

        # print("\t--- ", self.wing2, " ---")
        # "x": data.get("wing2_x"),
        # "y": data.get("wing2_y"),
        # "z": data.get("wing2_z"),
        # "airfoil_clmax": data.get("wing2_clmax_airfoil"),
        # "airfoil1_name": data.get("wing2_airfoil1_name"),
        # "airfoil2_name": data.get("wing2_airfoil2_name"),
        # "airfoil3_name": data.get("wing2_airfoil3_name"),
        # "span1": data.get("wing2_span1"),
        # "span2": data.get("wing2_span2"),
        # "chord1": data.get("wing2_chord1"),
        # "chord2": data.get("wing2_chord2"),
        # "chord3": data.get("wing2_chord3"),
        # "twist1": data.get("wing2_twist1"),
        # "twist2": data.get("wing2_twist2"),
        # "twist3": data.get("wing2_twist3"),
        # "incidence": data.get("wing2_incidence"),
        # "CM_ca": data.get("wing2_CM_ca"),
        # print()

        # print("\t--- ", self.hs, " ---")
        # "x": data.get("hs_x"),
        # "y": data.get("hs_y"),
        # "z": data.get("hs_z"),
        # "airfoil_clmax": data.get("hs_clmax_airfoil"),
        # "airfoil1_name": data.get("hs_airfoil1_name"),
        # "airfoil2_name": data.get("hs_airfoil2_name"),
        # "airfoil3_name": data.get("hs_airfoil3_name"),
        # "span1": data.get("hs_span1"),
        # "span2": data.get("hs_span2"),
        # "chord1": data.get("hs_chord1"),
        # "chord2": data.get("hs_chord2"),
        # "chord3": data.get("hs_chord3"),
        # "twist1": data.get("hs_twist1"),
        # "twist2": data.get("hs_twist2"),
        # "twist3": data.get("hs_twist3"),
        # "incidence": data.get("hs_incidence"),
        # "CM_ca": data.get("hs_CM_ca"),
        # print()

        # "x": data.get("motor_x"),
        # "y": data.get("motor_y"),
        # "z": data.get("motor_z"),
        # "static_thrust": data.get("static_thrust"),
        # "linear_decay_coefficient": data.get("linear_decay_coefficient")
        # print()

        # "x": data.get("cg_x"),
        # "z": data.get("cg_z")
        # print()

        # "x": data.get("tpr_x"),
        # "z": data.get("tpr_z")
        # print()
        # print('-------------------------------------------------------------')



from ADR import parameters
from ADR.Components.Plane import Plane
from ADR.Analysis.Performance.Power import Power

from matplotlib import pyplot as plt


def plot_power_curves(power_analysis):

    fig1, ((ax1, ax2, ax3, ax4)) = plt.subplots(1, 4)

    ax1.plot(power_analysis.alpha_df, label="Alpha")
    ax1.grid()
    ax1.legend()

    ax2.plot(power_analysis.thrust_required_df, label="Thrust required")
    ax2.plot(power_analysis.thrust_available_df, label="Thrust available")
    ax2.grid()
    ax2.legend()

    ax3.plot(power_analysis.power_required_df, label="Power required")
    ax3.plot(power_analysis.power_available_df, label="Power available")
    ax3.grid()
    ax3.legend()

    ax4.plot(power_analysis.power_excess_df, label="Power excess")
    ax4.plot(power_analysis.power_excess_df, label="Power excess")
    ax4.grid()
    ax4.legend()


def power_curves(mtow, plot=True):
    plane_parameters = parameters.get_plane_parameters()
    performance_data = parameters.get_performance_parameters()

    plane = Plane(plane_parameters)
    plane.mtow = mtow
    power_analysis = Power(plane, performance_data)
    plane.show_plane()

    if plot == True:
        print("V_max : ", plane.V_max)
        print("V_min : ", plane.V_min)
        plot_power_curves(power_analysis)
        plt.show()


if __name__ == "__main__":
    power_curves(mtow=18.299999999999994)



import pandas as pd
import numpy as np


from ADR.Core.import_functions import (
    import_airfoil_aerodynamic_data,
    import_airfoil_coordinates,
)


class Airfoil:
    def __init__(self, data):
        self.name = data.get("airfoil_name")

        self.Cl_alpha, self.Cd_alpha, self.Cm_alpha = import_airfoil_aerodynamic_data(
            self.name
        )
        self.airfoil_x_coords, self.airfoil_y_coords = import_airfoil_coordinates(
            self.name
        )

        self.import_camber_line()

        self.generate_upper_surface_coordinates()
        self.generate_inner_surface_coordinates()
        self.calc_perimeter()
        self.calc_area()

    def __str__(self):
        return self.name

    def get_Cl(self, alpha):
        Cl = np.interp(alpha, self.Cl_alpha.index.values, self.Cl_alpha["Cl"])
        return Cl

    def get_Cd(self, alpha):
        Cd = np.interp(alpha, self.Cd_alpha.index.values, self.Cd_alpha["Cd"])
        return Cd

    def get_Cm(self, alpha):
        Cm = np.interp(alpha, self.Cm_alpha.index.values, self.Cm_alpha["Cm"])
        return Cm

    def generate_upper_surface_coordinates(self):

        self.n = self.airfoil_x_coords.size
        # número de elementos do vetor de coordenadas horizontais

        self.minpos = np.argmin(self.airfoil_x_coords)
        # posição do menor valor no vetor de coordenadas horizontais,
        # determina a posição do bordo de ataque

        self.airfoil_x_coords_ext = np.flip(
            self.airfoil_x_coords[0: self.minpos])
        # coordenadas horizontais do extradorso

        self.airfoil_y_coords_ext = np.flip(
            self.airfoil_y_coords[0: self.minpos])
        # coordenadas verticais do extradorso

        self.n_ext = self.airfoil_x_coords_ext.size
        # número de elementos do vetor de coordenadas horizontais do extradorso

    def generate_inner_surface_coordinates(self):

        self.airfoil_x_coords_int = self.airfoil_x_coords[
            self.minpos: self.n - 1
        ]
        # coordenadas horizontais do intradorso

        self.airfoil_y_coords_int = self.airfoil_y_coords[
            self.minpos: self.n - 1
        ]
        # coordenadas verticais do intradorso

        self.n_int = (self.airfoil_x_coords_int.size)
        # número de elementos do vetor de coordenadas horizontais do intradorso

    def calc_perimeter(self):

        self.delta_perimeter_array_ext = np.empty([self.n_ext - 1])
        # vetor contando as parcelas de perimetro do extradorso

        self.delta_perimeter_array_int = np.empty([self.n_int - 1])
        # vetor contando as parcelas de perimetro do intradorso

        for i in range(0, self.n_ext - 2):
            delta_perimeter_i_ext = (
                (self.airfoil_x_coords_ext[i + 1]
                 - self.airfoil_x_coords_ext[i]) ** 2
                + (self.airfoil_y_coords_ext[i + 1]
                    - self.airfoil_y_coords_ext[i]) ** 2
            ) ** 0.5
            self.delta_perimeter_array_ext[i] = delta_perimeter_i_ext
        perimeter_ext = np.sum(self.delta_perimeter_array_ext)
        # perímetro do extradorso do perfil

        for i in range(0, self.n_int - 2):
            delta_perimeter_i_int = (
                (self.airfoil_x_coords_int[i + 1]
                 - self.airfoil_x_coords_int[i]) ** 2
                + (self.airfoil_y_coords_int[i + 1]
                    - self.airfoil_y_coords_int[i]) ** 2
            ) ** 0.5
            self.delta_perimeter_array_int[i] = delta_perimeter_i_int
        perimeter_int = np.sum(self.delta_perimeter_array_int)
        # perímetro do intradorso do perfil

        self.perimeter = perimeter_ext + perimeter_int
        # perimetro total do perfil

        self.sum_perimeter_ext_pos = np.empty([self.n_ext - 1])
        sum_perimeter_ext_pos_i = 0
        for i in range(0, self.n_ext - 2):
            sum_perimeter_ext_pos_i = (
                sum_perimeter_ext_pos_i + self.delta_perimeter_array_ext[i]
            )
            self.sum_perimeter_ext_pos[i] = sum_perimeter_ext_pos_i

        self.sum_perimeter_int_pos = np.empty([self.n_int - 1])
        sum_perimeter_int_pos_i = 0
        for i in range(0, self.n_int - 2):
            sum_perimeter_int_pos_i = (
                sum_perimeter_int_pos_i + self.delta_perimeter_array_int[i]
            )
            self.sum_perimeter_int_pos[i] = sum_perimeter_int_pos_i

    def calc_area(self):

        self.delta_area_array_ext = np.empty([self.n_ext - 1])
        # vetor contando as parcelas de área
        # determinadas pelo extradordo e a linha de corda

        self.delta_area_array_int = np.empty([self.n_int - 1])
        # vetor contando as parcelas de área
        # determinadas pelo intradordo e a linha de corda

        for i in range(0, self.n_ext - 2):
            delta_area_ext_i = (
                (self.airfoil_y_coords_ext[i]
                 + self.airfoil_y_coords_ext[i + 1])
                * (self.airfoil_x_coords_ext[i + 1] - self.airfoil_x_coords_ext[i])
                * 0.5
            )
            self.delta_area_array_ext[i] = delta_area_ext_i
        area_ext = np.sum(self.delta_area_array_ext)
        # área determinada pelo extradordo e a linha de corda
        for i in range(0, self.n_int - 2):
            delta_area_int_i = (
                (self.airfoil_y_coords_int[i]
                 + self.airfoil_y_coords_int[i + 1])
                * (self.airfoil_x_coords_int[i + 1] - self.airfoil_x_coords_int[i])
                * 0.5
            )
            self.delta_area_array_int[i] = delta_area_int_i
        area_int = np.sum(self.delta_area_array_int)
        # área determinada pelo intradorso e a linha de corda

        self.area = area_ext - area_int
        # área do perfil

    def import_camber_line(self):

        np_array_x, np_array_y = import_airfoil_coordinates(self.name)
        x_interp = np.arange(0, 1, 0.02)
        minpos = np.argmin(np_array_x)
        # posição do menor valor no vetor de coordenadas horizontais,
        # determina a posição do bordo de ataque

        n = np_array_x.size
        # número de elementos do vetor de coordenadas horizontais

        np_array_x_ext = np_array_x[0:minpos]
        np_array_x_ext = np.flip(np_array_x_ext)
        # coordenadas horizontais do extradorso

        np_array_x_int = np_array_x[minpos: n - 1]
        # coordenadas horizontais do intradorso

        np_array_y_int = np_array_y[minpos: n - 1]
        # coordenadas verticais do intradorso

        np_array_y_ext = np_array_y[0:minpos]
        np_array_y_ext = np.flip(np_array_y_ext)
        # coordenadas verticais do extradorso

        y_interp_ext = np.interp(x_interp, np_array_x_ext, np_array_y_ext)
        y_interp_int = np.interp(x_interp, np_array_x_int, np_array_y_int)
        y_camber = (y_interp_int + y_interp_ext) / 2
        camber_line = pd.DataFrame({"x": x_interp, "y": y_camber})
        self.Camber_line = camber_line



import numpy as np
from ADR.Core.data_manipulation import dict_to_dataframe
from ADR.Core.data_manipulation import find_df_roots

from ADR.Methods.FundamentalEquations import drag


class Power:
    def __init__(self, plane, performance_parameters):

        self.plane = plane
        self.wing1 = plane.wing1
        self.wing2 = plane.wing2
        self.hs = plane.hs
        self.area_ref = plane.wing1.area
        self.rho = performance_parameters.get("rho_air")
        self.checks_and_update_mtow()

    def checks_and_update_mtow(self):
        self.plane.get_V_stall(self.rho)
        self.plane.get_V_CLmin(self.rho)
        self.velocity_range = np.arange(
            self.plane.V_stall, self.plane.V_CLmin, 0.1)

        self.power_available()
        self.power_required()
        self.power_excess()

        positive_power = self.power_excess_df["Power excess"] > 0
        has_power_excess = positive_power.any()

        while has_power_excess == False and self.plane.mtow != 0:
            positive_power = self.power_excess_df["Power excess"] > 0
            has_power_excess = positive_power.any()
            # TODO: This is a big reduce-step. We should get this down by getting the power analysis time down.
            self.plane.mtow -= 1
            print("New MTOW: {}".format(self.plane.mtow))
            if self.plane.mtow > 0:
                self.power_available()
                self.power_required()
                self.power_excess()
            else:
                self.plane.mtow = 0
                print("Aircraft cannot sustain flight even with zero weight")

        self.get_V_min_max()

    def power_required(self):
        thrust_required_dict = {}
        power_required_dict = {}
        alpha_dict = {}
        for velocity in self.velocity_range:
            total_lift = 0
            alpha = self.plane.stall_min
            while total_lift < self.plane.mtow * 9.81:
                alpha += 0.1
                total_lift = self.wing1.lift(self.rho, velocity, alpha) - self.hs.lift(
                    self.rho, velocity, alpha
                )
                if self.plane.plane_type == "biplane":
                    total_lift += self.wing2.lift(self.rho, velocity, alpha)
                if alpha >= self.plane.stall_max:
                    alpha_nivel = None
                    break
                else:
                    alpha_nivel = alpha

            thrust_required = (
                drag(self.rho, velocity, self.plane.S_tp, self.plane.CD_tp)
                + drag(self.rho, velocity, self.plane.S_fus, self.plane.CD_fus)
                + self.wing1.drag(self.rho, velocity, alpha_nivel)
                + self.wing2.drag(self.rho, velocity, alpha_nivel)
                + self.hs.drag(self.rho, velocity, alpha_nivel)
            )

            alpha_dict[velocity] = alpha_nivel
            thrust_required_dict[velocity] = thrust_required

        for velocity in thrust_required_dict:
            power_required_dict[velocity] = thrust_required_dict[velocity] * velocity

        self.thrust_required_dict = thrust_required_dict
        self.power_required_dict = power_required_dict
        self.alpha_dict = alpha_dict
        self.alpha_df = dict_to_dataframe(alpha_dict, "Alpha", "Velocity")
        self.thrust_required_df = dict_to_dataframe(
            thrust_required_dict, "Thrust required", "Velocity"
        )
        self.power_required_df = dict_to_dataframe(
            power_required_dict, "Power required", "Velocity"
        )

        return self.alpha_df, self.thrust_required_df, self.power_required_df

    def power_available(self):
        thrust_available_dict = {}
        power_available_dict = {}

        for velocity in self.velocity_range:
            thrust_available = self.plane.motor.thrust(velocity)
            thrust_available_dict[velocity] = thrust_available

        for velocity in thrust_available_dict:
            power_available_dict[velocity] = thrust_available_dict[velocity] * velocity

        self.thrust_available_dict = thrust_available_dict
        self.power_available_dict = power_available_dict

        self.thrust_available_df = dict_to_dataframe(
            thrust_available_dict, "Thrust available", "Velocity"
        )
        self.power_available_df = dict_to_dataframe(
            power_available_dict, "Power available", "Velocity"
        )

        return self.thrust_available_df, self.power_available_df

    def power_excess(self):
        power_excess_dict = {}
        for velocity in self.power_available_dict:
            power_required = self.power_required_dict[velocity]
            power_available = self.power_available_dict[velocity]
            power_excess_dict[velocity] = power_available - power_required
        self.power_excess_dict = power_excess_dict
        self.power_excess_df = dict_to_dataframe(
            power_excess_dict, "Power excess", "Velocity"
        )

    def get_V_min_max(self):
        roots = find_df_roots(self.power_excess_df, "Power excess")
        if len(roots) == 1:
            self.plane.V_min = self.plane.V_stall
            self.plane.V_max = roots[0]
            alpha_max = self.alpha_df.max()[0]
        elif len(roots) == 2:
            self.plane.V_min = roots[0]
            self.plane.V_max = roots[1]
            alpha_max = np.interp(
                self.plane.V_min, self.alpha_df.index.values, self.alpha_df["Alpha"]
            )
        elif len(roots) == 0:
            self.plane.V_min = self.plane.V_stall
            self.plane.V_max = np.amax(self.velocity_range)
            alpha_max = self.alpha_df.max()[0]
        self.plane.alpha_min = self.alpha_dict[self.plane.V_max]

        print("Alpha_max: {}".format(alpha_max))
        self.plane.alpha_max = alpha_max



class TrimmRangeChecker:
    def __init__(self, plane):
        self.plane = plane
        self.alpha_min_trimm = plane.alpha_trimm_min
        self.alpha_max_trimm = plane.alpha_trimm_max
        self.alpha_min_power = plane.alpha_min
        self.alpha_max_power = plane.alpha_max

        self.trimm_tolerance = 2

    def check(self):
        if abs(self.alpha_min_power + self.trimm_tolerance) <= abs(
            self.alpha_min_trimm
        ):
            self.plane.trimm_for_low_angles = True
        else:
            self.plane.trimm_for_low_angles = False
            print("This aircraft does *not* trimm for low angles!")

        if abs(self.alpha_max_power - self.trimm_tolerance) <= abs(
            self.alpha_max_trimm
        ):
            self.plane.trimm_for_high_angles = True
        else:
            self.plane.trimm_for_high_angles = False
            print("This aircraft does *not* trimm for high angles!")



from ADR.Components.Component import Component


class CA(Component):
    def __init__(self, data):
        super().__init__(data)

        self.abs_x = data.get("surface_x") + data.get("x")
        self.abs_z = data.get("surface_z") + data.get("z")



class StaticMarginChecker:
    def __init__(self, plane):
        self.plane = plane
        self.SM_alpha = plane.SM_alpha

    def check(self):
        positive_alphas = self.SM_alpha[(self.SM_alpha.index > 0)]
        positive_sm = positive_alphas["SM"] > 0

        if positive_sm.all() == True:
            self.plane.positive_sm_for_positive_alphas = True
        else:
            self.plane.positive_sm_for_positive_alphas = False
            print(
                "This aircraft does *not* have positive Static Margin for all positive alphas!"
            )



from ADR.Methods.FundamentalEquations import lift, drag, moment
import numpy.testing as npt


def test_lift():
    npt.assert_almost_equal(lift(1.25, 14.3, 2.7, 0.9), 310.569, decimal=3)


def test_drag():
    npt.assert_almost_equal(drag(1.25, 14.3, 2.7, 0.5), 172.538, decimal=3)


def test_moment():
    npt.assert_almost_equal(moment(1.25, 14.3, 2.7, 0.6), 207.046, decimal=3)



from ADR.Components.Points.CA import CA
from ADR.Components.Aerodynamic_components.Aerodynamic_section import (
    Aerodynamic_section,
)
from ADR.Components.Component import Component
from ADR.Methods.VLM.AVL.avlwrapper_io import get_aero_coefs
import numpy as np
from math import radians, cos, sin


class Aerodynamic_surface(Component):
    def __init__(self, data):
        super().__init__(data)

        self.data = data

        self.airfoil_clmax = data.get("airfoil_clmax")
        self.airfoil1_name = data.get("airfoil1_name")
        self.airfoil2_name = data.get("airfoil2_name")
        self.airfoil3_name = data.get("airfoil3_name")
        self.span1 = data.get("span1")
        self.span2 = data.get("span2")
        self.chord1 = data.get("chord1")
        self.chord2 = data.get("chord2")
        self.chord3 = data.get("chord3")
        self.twist1 = data.get("twist1")
        self.twist2 = data.get("twist2")
        self.twist3 = data.get("twist3")
        self.incidence = data.get("incidence")

        self.attack_angle = None
        self.CM_alpha_CG = None

        data_section1 = {
            "airfoil1_name": self.airfoil1_name,
            "airfoil2_name": self.airfoil2_name,
            "span": self.span1,
            "chord1": self.chord1,
            "chord2": self.chord2,
            "twist1": self.twist1,
            "twist2": self.twist2,
        }

        data_section2 = {
            "airfoil1_name": self.airfoil2_name,
            "airfoil2_name": self.airfoil3_name,
            "span": self.span2,
            "chord1": self.chord2,
            "chord2": self.chord3,
            "twist1": self.twist2,
            "twist2": self.twist3,
        }

        self.section1 = Aerodynamic_section(data_section1)
        self.section2 = Aerodynamic_section(data_section2)

        self.area = 2 * (self.section1.area + self.section2.area)
        self.MAC = self.calc_MAC()

        self.vlm = "AVL"
        self.calc_aerodynamic_data()

        self.ca = CA(
            {"x": -self.MAC / 4, "z": 0, "surface_x": self.x, "surface_z": self.z}
        )

    def __str__(self):
        return self.__class__.__name__

    def update_alpha(self, alpha_airplane):
        self.attack_angle = alpha_airplane + self.incidence

    def calc_aerodynamic_data(self):
        # This entire method is NOT bullshit\

        if self.vlm == "AVL":
            a, b, c, self.CL_alpha, self.CD_alpha, self.Cm_alpha = get_aero_coefs(
                self.data, self.airfoil_clmax)

            self.CM_ca = self.Cm_alpha["Cm"].mean()

            self.stall_min = self.CL_alpha.index.min()
            self.stall_max = self.CL_alpha.index.max()

        self.dCL_dalpha = self.CL_alpha.diff()
        self.dCD_dalpha = self.CD_alpha.diff()
        self.dCL_dalpha.fillna(method="bfill", inplace=True)
        self.dCD_dalpha.fillna(method="bfill", inplace=True)

        self.downwash_angle = 0

    def moment_on_CG(self, reference_surface, cg, alpha_plane):

        resultant = 0

        surface_CL = self.CL_alpha.at[self.attack_angle, "CL"]
        surface_CD = self.CD_alpha.at[self.attack_angle, "CD"]

        sin_component = sin(radians(alpha_plane + self.incidence))
        cos_component = cos(radians(alpha_plane + self.incidence))

        horizontal_distance = self.x + self.ca.x - cg.x
        vertical_distance = self.z + self.ca.z - cg.z

        item1 = surface_CL * cos_component * horizontal_distance / reference_surface.MAC
        item2 = surface_CL * sin_component * vertical_distance / reference_surface.MAC
        item3 = surface_CD * sin_component * horizontal_distance / reference_surface.MAC
        item4 = surface_CD * cos_component * vertical_distance / reference_surface.MAC
        if self.__str__() == "Wing":
            resultant = +item1 - item2 + item3 + item4
        elif self.__str__() == "HS":
            resultant = -item1 + item2 + item3 + item4

        CM = (
            (self.CM_ca * self.MAC / reference_surface.MAC + resultant)
            * self.area
            / reference_surface.area
        )
        return CM

    def get_alpha_range(self):
        alpha_range = np.arange(self.stall_min, self.stall_max + 1)
        return alpha_range

    def calc_MAC(self):
        MAC = self.section1.MAC * (
            self.section1.area / (self.section1.area + self.section2.area)
        ) + self.section2.MAC * (
            self.section2.area / (self.section1.area + self.section2.area)
        )
        return MAC

    def get_CL(self, alpha):
        CL = np.interp(alpha, self.CL_alpha.index.values, self.CL_alpha["CL"])
        return CL

    def get_CD(self, alpha):
        CD = np.interp(alpha, self.CD_alpha.index.values, self.CD_alpha["CD"])
        return CD

    def get_CM(self):
        return self.CM_ca

    def lift(self, air_density, velocity, alpha):
        lift = 0.5 * air_density * velocity ** 2 * \
            self.area * self.get_CL(alpha)
        return lift

    def drag(self, air_density, velocity, alpha):
        drag = 0.5 * air_density * velocity ** 2 * \
            self.area * self.get_CD(alpha)
        return drag

    def moment(self, air_density, velocity, alpha):
        moment = (
            0.5 * air_density * velocity ** 2 * self.area * self.MAC * self.get_CM()
        )
        return moment



from ADR import parameters
from ADR.Components.Plane import Plane
from ADR.Analysis.Performance.Takeoff import Takeoff
from matplotlib import pyplot as plt


def plot_takeoff_data(takeoff_analysis, mtow):

    fig1, ((ax1, ax2, ax3), (ax4, ax5, ax6)) = plt.subplots(2, 3)

    ax1.plot(takeoff_analysis.mass_dict[mtow]["N"], label="Normal")
    ax1.plot(takeoff_analysis.mass_dict[mtow]["L"], label="Lift")
    ax1.plot(takeoff_analysis.mass_dict[mtow]["L_w1"], label="Lift Wing1")
    ax1.plot(takeoff_analysis.mass_dict[mtow]["L_w2"], label="Lift Wing2")
    ax1.plot(takeoff_analysis.mass_dict[mtow]["L_hs"], label="Lift HS")
    ax1.grid()
    ax1.legend()

    ax2.plot(takeoff_analysis.mass_dict[mtow]["D"], label="Drag")
    ax2.plot(takeoff_analysis.mass_dict[mtow]["D_w1"], label="Drag Wing1")
    ax2.plot(takeoff_analysis.mass_dict[mtow]["D_w2"], label="Drag Wing2")
    ax2.plot(takeoff_analysis.mass_dict[mtow]["D_hs"], label="Drag HS")
    ax2.grid()
    ax2.legend()

    ax3.plot(takeoff_analysis.mass_dict[mtow]["M"], label="Moment")
    ax3.plot(takeoff_analysis.mass_dict[mtow]["M_w1"], label="Moment Wing1")
    ax3.plot(takeoff_analysis.mass_dict[mtow]["M_w2"], label="Moment Wing2")
    ax3.plot(takeoff_analysis.mass_dict[mtow]["M_hs"], label="Moment HS")
    ax3.grid()
    ax3.legend()

    ax4.plot(takeoff_analysis.mass_dict[mtow]["dTheta"], label="dTheta")
    ax4.grid()
    ax4.legend()

    ax5.plot(takeoff_analysis.mass_dict[mtow]
             ["incidence_hs"], label="HS incidence")
    ax5.plot(takeoff_analysis.mass_dict[mtow]["theta"], label="Theta")
    ax5.grid()
    ax5.legend()

    ax6.plot(takeoff_analysis.mass_dict[mtow]["dist_x"], label="Distance")
    ax6.plot(takeoff_analysis.mass_dict[mtow]["V_x"], label="Velocity")
    ax6.grid()
    ax6.legend()


def takeoff(plot=True):
    plane_parameters = parameters.get_plane_parameters()
    performance_data = parameters.get_performance_parameters()

    plane = Plane(plane_parameters)
    takeoff_analysis = Takeoff(plane, performance_data)
    mtow = takeoff_analysis.get_mtow()

    print("Final MTOW is {}".format(mtow))
    print("V_takeoff : ", plane.V_takeoff)

    if plot == True:
        plot_takeoff_data(takeoff_analysis, mtow)
        plt.show()


if __name__ == "__main__":
    takeoff()



from ADR.Components.Aerodynamic_components.Airfoil import Airfoil


class Aerodynamic_section:
    def __init__(self, data):

        self.airfoil1_name = data.get("airfoil1_name")
        self.airfoil2_name = data.get("airfoil2_name")
        self.span = data.get("span")
        self.chord1 = data.get("chord1")
        self.chord2 = data.get("chord2")
        self.twist1 = data.get("twist1")
        self.twist2 = data.get("twist2")

        self.airfoil1 = Airfoil(data={"airfoil_name": self.airfoil1_name})
        self.airfoil2 = Airfoil(data={"airfoil_name": self.airfoil2_name})
        self.area = self.calc_area()
        self.MAC = self.calc_MAC()

    def calc_area(self):
        return (self.chord1 + self.chord2) * self.span / 2

    def calc_MAC(self):
        return self.chord1 - (
            2
            * (self.chord1 - self.chord2)
            * (0.5 * self.chord1 + self.chord2)
            / (3 * (self.chord1 + self.chord2))
        )



import numpy.testing as npt
from ADR.Components.Component import Component


def test_Component():

    component_parameters = {"x": 1, "y": 2, "z": 3}

    component_test = Component(component_parameters)

    if component_test.x == 1 and component_test.y == 2 and component_test.z == 3:
        check_test = 1

    npt.assert_almost_equal(check_test, 1)



class MaybeAnAssassin:
    def __init__(self, plane):
        self.plane = plane

    def score_or_kill(self):
        if (
            self.plane.trimm_for_low_angles
            and self.plane.trimm_for_high_angles
            and self.plane.positive_sm_for_positive_alphas
            and self.plane.dimensions_are_good
            and self.plane.mtow > 0
        ):

            self.plane.payload = self.plane.mtow - self.plane.dead_weight

            self.plane.score = 12.5 * self.plane.payload
            print("Plane is alive and scored {}".format(self.plane.score))
        else:
            self.plane.dead = True
            self.plane.score = 0
            print("Plane is dead")



import ADR
import pandas as pd


def get_filepath(folder, filename):

    package_filepath = ADR.__file__.replace("__init__.py", "")
    complete_filepath = package_filepath + folder + filename

    return complete_filepath


def import_airfoil_aerodynamic_data(airfoil_name):

    airfoil_aerodynamic_data_filepath = get_filepath(
        "World/Profiles/AerodynamicData/", "xf-" + airfoil_name + "-il-200000.csv"
    )

    airfoil_df = pd.read_csv(
        airfoil_aerodynamic_data_filepath, skiprows=10, index_col=0
    )
    Cl_alpha = airfoil_df[["Cl"]]
    Cd_alpha = airfoil_df[["Cd"]]
    Cm_alpha = airfoil_df[["Cm"]]

    return Cl_alpha, Cd_alpha, Cm_alpha


def import_airfoil_coordinates(airfoil_name):

    # Aqui o arquivo de coordenadas no formato de arquivo de texto, conforme copiado e colado de um banco de dados é lido e editado
    # removendo espaços em branco adicionais para facilitar a separação em data frames. O formato do arquivo de coordenadas deve ser
    # o formato selig.

    airfoil_coordinates_filepath = get_filepath(
        "World/Profiles/Coordinates/", airfoil_name + ".dat"
    )

    file1 = open(airfoil_coordinates_filepath, "r")
    file2 = open(airfoil_coordinates_filepath.replace(
        ".dat", "_edited.dat"), "wt")
    file2.write("x y \n")
    for line in file1:
        file2.write(" ".join(line.split()) + "\n")
    file1.close()
    file2.close()

    # -----Criação do dataframe e transformação de dataframe em vetores (listas)-----
    # A partir do arquivo de texto é criado o dataframe com as coordenadas dos pontos do perfil. Esse dataframe é separado em coordendas
    # horizontais e verticais e em seguida transformados em vetores (listas).

    df = pd.read_csv(
        airfoil_coordinates_filepath.replace(".dat", "_edited.dat"),
        sep=" ",
        skiprows=[1],
    )
    df_array_x = df.loc[:, "x"]
    df_array_y = df.loc[:, "y"]

    return (
        df_array_x.values,
        df_array_y.values,
    )  # vetores de coordenadas horizontais e verticais



from ADR.Components.Aerodynamic_components.Aerodynamic_surface import (
    Aerodynamic_surface,
)
import numpy as np


class HS(Aerodynamic_surface):
    def __init__(self, data):
        super().__init__(data)

        self.calc_area()

    def calc_area(self):
        self.area = 2 * (self.section1.area + self.section2.area)

    def update_alpha(self, alpha_airplane):
        self.attack_angle = self.incidence - alpha_airplane

    def set_incidence_range(self, alpha_plane_min, alpha_plane_max):
        self.incidence_range = np.arange(
            alpha_plane_min + self.stall_min, alpha_plane_max + self.stall_max + 1
        )



import ADR
from ADR import parameters

if __name__ == "__main__":
    plane_parameters = parameters.get_plane_parameters()
    performance_parameters = parameters.get_performance_parameters()

    ADR.analyzer(plane_parameters, performance_parameters, plot=True)



def lift(air_density, velocity, area, lift_coefficient):
    lift = 0.5 * air_density * velocity ** 2 * area * lift_coefficient
    return lift


def drag(air_density, velocity, area, drag_coefficient):
    drag = 0.5 * air_density * velocity ** 2 * area * drag_coefficient
    return drag


def moment(air_density, velocity, area, moment_coefficient):
    moment = 0.5 * air_density * velocity ** 2 * area * moment_coefficient
    return moment



from ADR.Components.Component import Component


class Motor(Component):
    def __init__(self, data):
        super().__init__(data)

        self.static_thrust = data.get("static_thrust")
        self.linear_decay_coefficient = data.get("linear_decay_coefficient")

    def thrust(self, velocity):
        thrust = self.static_thrust - self.linear_decay_coefficient * velocity
        return thrust



import pandas as pd
import numpy as np
import csv
import os


def dict_to_dataframe(dict, column_name, index_name):
    dataframe = pd.DataFrame.from_dict(
        dict, orient="index", columns=[column_name])
    dataframe.index.name = index_name
    return dataframe


def find_df_roots(df, column_name):

    roots = []

    new_df = df.fillna(method="ffill")
    new_df = new_df.fillna(method="bfill")

    index_array = new_df.index.values
    values_array = new_df[column_name].values

    sign_change_array = (np.diff(np.sign(values_array)) != 0) * 1
    # sign_change_array = np.where(np.diff(np.sign(values_array)))[0]

    for index, element in enumerate(sign_change_array):
        if element == 1:
            roots.append(index_array[index])
    return roots


def save_dict(plane_params, perf_params, mtow, state):
    dir_name = "saved_planes"
    dir_path = os.path.dirname(os.path.abspath(
        __file__)).replace("Core", "") + dir_name
    print(dir_path)
    filename = "plane_mtow_"
    filepath = dir_path + "/" + filename + str(round(mtow)) + "_" + state + "-"

    i = 0
    if os.path.isfile(filepath + str(i) + ".csv"):
        already_exist = True
        while already_exist:
            i += 1
            if not os.path.isfile(filepath + str(i) + ".csv"):
                already_exist = False

    filepath = filepath + str(i) + ".csv"
    with open(filepath, "a", newline="") as f:
        w = csv.writer(f)
        for key, val in plane_params.items():
            w.writerow([key, val])
        for key, val in perf_params.items():
            w.writerow([key, val])



from matplotlib import pyplot as plt


def plot_aerodynamic_data(plane):

    fig1, ((ax1, ax2, ax3, ax4)) = plt.subplots(1, 4)

    ax1.set_xlabel("Alpha")
    ax1.set_ylabel("CL")
    ax1.set_title("CL x Alpha")
    ax1.plot(plane.CL_alpha)
    ax1.grid()

    ax2.set_xlabel("Alpha")
    ax2.set_ylabel("CD")
    ax2.set_title("CD x Alpha")
    ax2.plot(plane.CD_alpha)
    ax2.grid()

    ax3.set_xlabel("Alpha")
    ax3.set_ylabel("CL")
    ax3.set_title("CL x Alpha for surfaces")
    ax3.plot(plane.wing1.CL_alpha, label="Wing1")
    ax3.plot(plane.wing2.CL_alpha, label="Wing2")
    ax3.plot(plane.hs.CL_alpha, label="HS")
    ax3.grid()
    ax3.legend()

    ax4.set_xlabel("CD")
    ax4.set_ylabel("CL")
    ax4.set_title("CL x CD")
    ax4.plot(plane.CD_alpha["CD"], plane.CL_alpha["CL"])
    ax4.grid()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="AircraftDR",
    version="0.1",
    author="Ceu Azul Aeronaves",
    author_email="ceuazulufsc@gmail.com",
    description="This is Aircraft Design Resources",
    url="https://github.com/CeuAzul/ADR",
    packages=setuptools.find_packages(),
    include_package_data=True,
    setup_requires=['wheel'],
    install_requires=['avlwrapper', 'numpy',
                      'matplotlib', 'pandas', 'scipy'],
    tests_requires=['pytest'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



class Ruler:
    def __init__(self, plane):
        self.plane = plane

    def measure(self):
        wing1_span = 2 * (self.plane.wing1.span1 + self.plane.wing1.span2)
        wing2_span = 2 * (self.plane.wing2.span1 + self.plane.wing2.span2)
        hs_span = 2 * (self.plane.hs.span1 + self.plane.hs.span2)
        y_max = max(wing1_span, wing2_span, hs_span)
        x_max = self.plane.motor.x - self.plane.hs.x + self.plane.hs.chord1
        total_dimension = y_max + x_max
        self.plane.total_dimensions = total_dimension

        if total_dimension < 3.71:
            self.plane.dimensions_are_good = True
        else:
            self.plane.dimensions_are_good = False
            print(
                "Plane is *outside* the dimensions. Total: {}".format(
                    self.plane.total_dimensions
                )
            )



from ADR import analyzer
import numpy.testing as npt


def test_analyser():
    plane_parameters = {
        "plane_type": "biplane",
        "wing1_x": 0,
        "wing1_y": 0,
        "wing1_z": 0,
        "wing1_clmax_airfoil": 2.2,
        "wing1_airfoil1_name": "s1223",
        "wing1_airfoil2_name": "s1223",
        "wing1_airfoil3_name": "s1223",
        "wing1_span1": 0.8605,
        "wing1_span2": 0.3895,
        "wing1_chord1": 0.4,
        "wing1_chord2": 0.35,
        "wing1_chord3": 0.25,
        "wing1_twist1": 0,
        "wing1_twist2": 0,
        "wing1_twist3": 0,
        "wing1_incidence": 0,
        "wing2_x": 0,
        "wing2_y": 0,
        "wing2_z": 0.6,
        "wing2_clmax_airfoil": 2.2,
        "wing2_airfoil1_name": "s1223",
        "wing2_airfoil2_name": "s1223",
        "wing2_airfoil3_name": "s1223",
        "wing2_span1": 0.8605,
        "wing2_span2": 0.3895,
        "wing2_chord1": 0.4,
        "wing2_chord2": 0.35,
        "wing2_chord3": 0.25,
        "wing2_twist1": 0,
        "wing2_twist2": 0,
        "wing2_twist3": 0,
        "wing2_incidence": 0,
        "hs_x": -0.5928,
        "hs_y": 0,
        "hs_z": 0.1,
        "hs_clmax_airfoil": 1.55,
        "hs_airfoil1_name": "sd7037",
        "hs_airfoil2_name": "sd7037",
        "hs_airfoil3_name": "sd7037",
        "hs_span1": 0.8,
        "hs_span2": 0.5,
        "hs_chord1": 0.16,
        "hs_chord2": 0.14,
        "hs_chord3": 0.12,
        "hs_twist1": 0,
        "hs_twist2": 0,
        "hs_twist3": 0,
        "hs_incidence": 0,
        "vs_x": -0.7,
        "vs_y": 0,
        "vs_z": 0,
        "vs_clmax_airfoil": 2.2,
        "vs_airfoil1_name": "s1223",
        "vs_airfoil2_name": "s1223",
        "vs_airfoil3_name": "s1223",
        "vs_span1": 0.1,
        "vs_span2": 0.1,
        "vs_chord1": 0.2,
        "vs_chord2": 0.2,
        "vs_chord3": 0.1,
        "vs_twist1": 0,
        "vs_twist2": 0,
        "vs_twist3": 0,
        "vs_incidence": 0,
        "motor_x": 0.25,
        "motor_z": 0,
        "static_thrust": 45,
        "linear_decay_coefficient": 1.1,
        "cg_x": -0.103,
        "cg_z": -0.1,
        "tpr_x": -0.153,
        "tpr_z": -0.2,
        "Iyy_TPR": 0.2,
        "CD_tp": 0.8,
        "S_tp": 0.001,
        "CD_fus": 0.6,
        "S_fus": 0.02,
        "u_k": 0.05,
    }

    performance_parameters = {
        "rho_air": 1.1,  # Densidade do ar [kg/m^3]
        "dist_max": 45,  # Distancia maxima de decolagem pelo regulamento [m]
        # Distancia antes do fim da pista em que o piloto aciona o profundor [m]
        "offset_pilot": 5,
    }

    npt.assert_almost_equal(analyzer(
        plane_parameters, performance_parameters, plot=False), 193.749, decimal=3)



from ADR.Components.Plane import Plane
from ADR.Core.data_manipulation import save_dict
from ADR.Analysis.Performance.Takeoff import Takeoff
from ADR.Analysis.Performance.run_Takeoff import plot_takeoff_data
from ADR.Analysis.Performance.Power import Power
from ADR.Analysis.Performance.run_Power import plot_power_curves
from ADR.Analysis.Stability.FlightStability.FlightStability import FlightStability
from ADR.Analysis.Stability.FlightStability.run_FlightStability import (
    plot_stability_data,
)
from ADR.Analysis.Aerodynamics.AerodynamicPlots import plot_aerodynamic_data

from ADR.Checkers.TrimmRange import TrimmRangeChecker
from ADR.Checkers.StaticMargin import StaticMarginChecker
from ADR.Checkers.Scoreboard import MaybeAnAssassin
from ADR.Checkers.Dimensions import Ruler

from matplotlib import pyplot as plt

import traceback
import logging


def analyzer(plane_parameters, performance_parameters, plot=False):

    try:
        plane = Plane(plane_parameters)
        plane.show_plane()

        takeoff_analysis = Takeoff(plane, performance_parameters)
        mtow = takeoff_analysis.get_mtow()

        print("Initial MTOW is {}".format(mtow))

        flight_stability = FlightStability(plane)
        flight_stability.CM_plane_CG(plane.cg)
        flight_stability.static_margin()

        power_analysis = Power(plane, performance_parameters)

        trimm_range_checker = TrimmRangeChecker(plane)
        trimm_range_checker.check()

        sm_checker = StaticMarginChecker(plane)
        sm_checker.check()

        ruler = Ruler(plane)
        ruler.measure()

        maybe_an_assassin = MaybeAnAssassin(plane)
        maybe_an_assassin.score_or_kill()

        print("Final MTOW is {}".format(plane.mtow))

        if plot == True:
            plot_takeoff_data(takeoff_analysis, mtow)
            plot_stability_data(flight_stability)
            plot_power_curves(power_analysis)
            plot_aerodynamic_data(plane)
            plt.show()

        if plane.dead == True:
            save_dict(plane_parameters, performance_parameters, mtow, "dead")
        else:
            save_dict(plane_parameters, performance_parameters, mtow, "alive")
        return (plane.score,)

    except Exception:
        logging.error(traceback.format_exc())
        print(
            "-----------------------------------Error-----------------------------------"
        )
        return (0,)



# Generates dictionary with plane and takeoff data

plane_parameters = {
    "plane_type": "biplane",
    "wing1_x": 0,
    "wing1_y": 0,
    "wing1_z": 0,
    "wing1_clmax_airfoil": 2.2,
    "wing1_airfoil1_name": "s1223",
    "wing1_airfoil2_name": "s1223",
    "wing1_airfoil3_name": "s1223",
    "wing1_span1": 0.8605,
    "wing1_span2": 0.3895,
    "wing1_chord1": 0.4,
    "wing1_chord2": 0.35,
    "wing1_chord3": 0.25,
    "wing1_twist1": 0,
    "wing1_twist2": 0,
    "wing1_twist3": 0,
    "wing1_incidence": 0,
    "wing2_x": 0,
    "wing2_y": 0,
    "wing2_z": 0.6,
    "wing2_clmax_airfoil": 2.2,
    "wing2_airfoil1_name": "s1223",
    "wing2_airfoil2_name": "s1223",
    "wing2_airfoil3_name": "s1223",
    "wing2_span1": 0.8605,
    "wing2_span2": 0.3895,
    "wing2_chord1": 0.4,
    "wing2_chord2": 0.35,
    "wing2_chord3": 0.25,
    "wing2_twist1": 0,
    "wing2_twist2": 0,
    "wing2_twist3": 0,
    "wing2_incidence": 0,
    "hs_x": -0.5928,
    "hs_y": 0,
    "hs_z": 0.1,
    "hs_clmax_airfoil": 1.55,
    "hs_airfoil1_name": "sd7037",
    "hs_airfoil2_name": "sd7037",
    "hs_airfoil3_name": "sd7037",
    "hs_span1": 0.8,
    "hs_span2": 0.5,
    "hs_chord1": 0.16,
    "hs_chord2": 0.14,
    "hs_chord3": 0.12,
    "hs_twist1": 0,
    "hs_twist2": 0,
    "hs_twist3": 0,
    "hs_incidence": 0,
    "vs_x": -0.7,
    "vs_y": 0,
    "vs_z": 0,
    "vs_clmax_airfoil": 2.2,
    "vs_airfoil1_name": "s1223",
    "vs_airfoil2_name": "s1223",
    "vs_airfoil3_name": "s1223",
    "vs_span1": 0.1,
    "vs_span2": 0.1,
    "vs_chord1": 0.2,
    "vs_chord2": 0.2,
    "vs_chord3": 0.1,
    "vs_twist1": 0,
    "vs_twist2": 0,
    "vs_twist3": 0,
    "vs_incidence": 0,
    "motor_x": 0.25,
    "motor_z": 0,
    "static_thrust": 45,
    "linear_decay_coefficient": 1.1,
    "cg_x": -0.103,
    "cg_z": -0.1,
    "tpr_x": -0.153,
    "tpr_z": -0.2,
    "Iyy_TPR": 0.2,
    "CD_tp": 0.8,
    "S_tp": 0.001,
    "CD_fus": 0.6,
    "S_fus": 0.02,
    "u_k": 0.05,
}

performance_parameters = {
    "rho_air": 1.1,  # Densidade do ar [kg/m^3]
    "dist_max": 45,  # Distancia maxima de decolagem pelo regulamento [m]
    # Distancia antes do fim da pista em que o piloto aciona o profundor [m]
    "offset_pilot": 5,
}


def get_plane_parameters():
    return plane_parameters


def get_performance_parameters():
    return performance_parameters
